function projZ = dualhankelprojection(Z, options)
    % Project the Z matrix with antidiagonal summing to zero.
    [m, n] = size(Z);
    
    if ~isfield(options, 'hankelidx')
        options.hankelidx =  hankel(1:m,m:(n-1)+m);
        options.hankelnormalizer =  [1:m, m*ones(1,m + n - 1 - 2*m), m:-1:1]'; % Only for fat matrices, will be different for long matrices
    end
    
    
    % Mex code
    [~, projZ] = adjointHankelProjZmex(Z, uint32(options.hankelidx), options.hankelnormalizer);
    
    %     % Sanity checks
    %     % Matlab code
    %     adjointZ = adjointhankel(Z, m, n);
    %     c = adjointZ(1:m);
    %     r = adjointZ(m:end);
    %     hankel(c, r);
    %     projZ1 =  Z - hankel(c, r);
    %     norm(projZ1 - projZ, 'fro')
    
    
    
    
    %     % Sanity check. This should be zero as hankel(c, r) is the normal
    %     % component.
    %     trace(projZ'*hankel(c, r))
    %     pause;
    
end

% Adjoint of the Hankel operation.
function adjZ = adjointhankel(Z, m, n, Hankellinearindices)
    if ~exist('Hankellinearindices', 'var') || isempty(Hankellinearindices)
        Hankellinearindices = {};
        for t = 1:(m+n-1)
            [I, J] = getindices(t+1, m, n);
            idx_t = sub2ind([m n], I, J);
            Hankellinearindices{t} = idx_t;
        end
    end
    
    z = Z(:);
    adjZ = zeros(m + n -1, 1);
    for tt = 1:(n + m -1)
        idx_tt = Hankellinearindices{tt};
        adjZ(tt,1) = mean(z(idx_tt)); % BM: take the mean.
    end
end

