clear; clc; close all;
rng(25);


%% Fazel et al. data
% rcompletegen;
% d = i;
% T = j;
% y = bx';
% p_data = y;
% p0 = p_data;
% Y = hankel(y(1:i),y(i:end));
% truerank = 5; % An arbit value.




%% Data generation with truerank.
truerank = 5;

d = 10;
T = 10;
sys = drss(truerank);    % random system of order r, requires control toolbox
y0 = impulse(sys, d + T); % sequence that parameterizes Y
y = [y0(2:d + 1); y0(d + 2:d + T)];
p0 = y;

sigma = 0;
p_data  = p0 +  sigma*randn(length(y), 1);
y = p_data;
Y = hankel(y(1:d), y(d:end));


%% Rank that we impose.
rank = truerank;



%% Proposed algorithm.
% We reshape the Hankel matrix to size (rank+1)-by-(d+T-rank) and then
% learn a rank "rank" matrix.

C = 1e4; % A smaller C, e.g., 1e2, for noisy data. Else, use a large value like 1e5.

% Problem structure
problem.d = min(d, rank + 1);
problem.T = length(y) - problem.d + 1;
problem.r = rank;
problem.C = C;
problem.y = p_data;
problem.y_test = p0;


% Options are not mandatory
opts.maxiter = 100;
opts.tolgradnorm = 1e-6;
opts.maxinner = 20;
opts.tolrelgradnorm = 1e-6;
opts.method = 'tr';
opts.innerpcgtol = 1e-10;
opts.innerpcgmaxiter = 500;
opts.innerpcgdottol = 1e-6;
opts.innerpcgdotmaxiter = 500;
opts.preconreg = 1e-6;
opts.verbosity = 2;
opts.reg = 1e-10; % A low value.

% Initialization with SVD of hankel(problem.y). This works well as
% suggested by Markovsky and Usevich.
% For random initialization, Linit = [].
% [Linit, ~, ~] = svds(hankel(problem.y(1:problem.d), problem.y(problem.d : end)), problem.r);
% Linit = Linit/norm(Linit, 'fro');
Linit = [];

% Call the proposed algorithm
[Lopt, ~, ~, ~, infos, options] = fixedrank_hankelmatrixapproximation(problem, Linit,opts);

% Refining to get a more accurate low-rank solution.
opts = options;
opts.innerpcgtol = 1e-15;
opts.innerpcgmaxiter = 5000;
opts.preconreg = 1e-6;
opts.reg = 0;
[zopt, Sopt] = HankelsquareLoss_dual_get_zS(Lopt, problem.C, problem.y, opts);

% Optimal low-rank factors are Lopt and Ropt.
Ropt = Lopt'*Sopt;
Wopt = Lopt*Ropt;

% The optimal data vector.
wopt = [Wopt(1:problem.d,1); (Wopt(end,2:end))'];

% Check rank enforcing.
Y_proposed = hankel(wopt(1:d), wopt(d:end));
sv_proposed  = svd(Y_proposed);
sv_proposed(rank + 1)

% Compute the rmses.
true_rmse_proposed = sqrt(mean((wopt - problem.y_test).^2));
train_rmse_proposed = sqrt(mean((wopt - problem.y).^2));



%%
% Plot for relative duality gap
fs = 20;
figure;
semilogy([infos.rel_duality_gap],'-O','Color','b','LineWidth',2, 'MarkerSize',13);
ax1 = gca;
set(ax1,'FontSize',fs);
xlabel(ax1,'Iterations','FontSize',fs);
ylabel(ax1,'Relative duality gap','FontSize',fs);
legend('Proposed');
legend 'boxoff';
box off;


fs = 20;
figure;
semilogy([infos.loss_test_rmse],'-O','Color','b','LineWidth',2, 'MarkerSize',13);
ax1 = gca;
set(ax1,'FontSize',fs);
xlabel(ax1,'Iterations','FontSize',fs);
ylabel(ax1,'RMSE on the true data','FontSize',fs);
legend('Proposed');
legend 'boxoff';
box off;


fs = 20;
figure;
semilogy([infos.loss_rmse],'-O','Color','b','LineWidth',2, 'MarkerSize',13);
ax1 = gca;
set(ax1,'FontSize',fs);
xlabel(ax1,'Iterations','FontSize',fs);
ylabel(ax1,'RMSE on the noisy data','FontSize',fs);
legend('Proposed');
legend 'boxoff';
box off;