% This sample demo test code creates a synthetic matrix completion data set
% of dimensions dxT and calls our first-order algorithm,
% Proposed-cg-sq, to complete the matrix.

clear; clc; close all;
randomize_seed = 25;
rng(randomize_seed);
dataset = 'synthetic';

sp = 1;
fprintf('Dataset: %s, Split: %d of %d\n', dataset, sp, sp);
fprintf('randomize_seed: %d\n', randomize_seed);

% A synthetic problem instance.
d = 500; % 
T = 50000; % 
truerank = 5;
OS = 6; % Over sampling. No of known entries relative to dimension. Typically 4,5,8.
noiseFac = 0; 1e-6; % Noise factor % good config: 0
fprintf('d: %d, T: %d, truerank: %d, OS: %d, noiseFac: %e\n',d,T,truerank,OS,noiseFac);

% Call function to create synthetic data set
[I, J, Y, I_test, J_test, Y_test] = createSyntheticData(d, T, truerank, OS, noiseFac);
fprintf('Number of train instances: %d\n',length(Y));
fprintf('Number of test instances: %d\n',length(Y_test));

%% Create Problem structure and set hyper-parameters
% Set regularization parameter
C = 1e7;
fprintf('C: %e\n',C);
% Rank that we impose
rank = truerank;
fprintf('rank: %d\n',rank);

% Problem structure
problem.d = d;
problem.T = T;
problem.r = rank;
problem.C = C;
problem.I = I;
problem.J = J;
problem.Y = Y;
problem.I_test = I_test;
problem.J_test = J_test;
problem.Y_test = Y_test;


%% Proposed-cg-sq algorithm call

% Options are not mandatory
optscg.maxiter = 50;
optscg.tolgradnorm = 1e-5;% 1e-5
optscg.tolrelgradnorm = 1e-20;% 1e-20
optscg.method='cg'; % option which ensures call to Proposed-cg-sq
optscg.verbosity = 2; % 2 for full output.
optscg

fprintf('------------------ Proposed-cg-sq ------------------\n')

initproposedcgstart = tic;
% Initialization
mysparsemat = sparse(double(problem.I), double(problem.J), ones(length(problem.I), 1), problem.d, problem.T, length(problem.I));
mysetsparseentries(mysparsemat , problem.Y);
[Uinit, ~, ~]= svds(mysparsemat, problem.r);
Uinit = Uinit/norm(Uinit, 'fro');
initproposedcgtime = toc(initproposedcgstart);
fprintf('SVD based warm start\n');
fprintf('initproposedcgtime: %e\n',initproposedcgtime);

[Lopt, Ropt, Zopt, infoscg, optionscg] = fixedrank_matrixcompletion(problem,Uinit,optscg);
% Wopt = Lopt * Ropt; % uncomment this line in order to get the complete matrix
proposed_cg_plot_rmse = [infoscg.loss_test_rmse];
proposed_cg_plot_time = [infoscg.time]+initproposedcgtime;
proposed_cg_plot_dualitygap = [infoscg.rel_duality_gap];
