/*=================================================================
% Compile with: mex -lmwlapack -lmwblas -largeArrayDims nnmc_new_sparse_s.c
 *=================================================================*/

#include "mex.h"
#include "matrix.h"
#include "lapack.h"
#include "blas.h"
#include <math.h>
#include <sys/time.h>
#include <string.h> /* memset */
/*#include <omp.h>*/

/* Input Arguments */
#define	Ut	    		prhs[0]
#define ConstCell_Utx	prhs[1]
#define ConstMatrix     prhs[2]
#define UomegaUomegaCell prhs[3]
#define maxinnerepoch	prhs[4]
#define myTol			prhs[5]
#define maskS           prhs[6]
#define nnzS            prhs[7]

/*#define numThreads prhs[3]*/

/*#define defone 0*/
/*#define defone1 0*/

/* Timing functions */
struct timeval start()
{
    struct timeval tm ;
    gettimeofday(&tm, NULL);
    return tm ;
}

void stop(struct timeval tm1)
{
    struct timeval tm2;
    gettimeofday(&tm2, NULL);
    unsigned long long t = 1000 * (tm2.tv_sec - tm1.tv_sec) + (tm2.tv_usec - tm1.tv_usec) / 1000;
    printf("%llu ms\n", t);
}

unsigned long long stopAndGet(struct timeval tm1)
{
    struct timeval tm2;
    gettimeofday(&tm2, NULL);
    unsigned long long t = 1000 * (tm2.tv_sec - tm1.tv_sec) + (tm2.tv_usec - tm1.tv_usec) / 1000;
    return t ;
}

double stopAndGetDouble(struct timeval tm1)
{
    return (double)stopAndGet(tm1) ;
}

unsigned long long getMeanTimes(unsigned long long* times, int* nOfIterations, int maxthreads)
{
    unsigned long long meanTime = 0 ;
    int i = 0 ;
    for(i = 0 ; i < maxthreads ; i++)
    {
        if(nOfIterations[i] > 0)
            meanTime += times[i] ; /* /nOfIterations[i] ;   */
    }
    return meanTime ;
}


void initialize_vectormatrix_zero(double *B, size_t row, size_t col){
    size_t i, j;
    for (i=0;i<col;i++)
        for (j=0;j<row;j++)
            B[i*row+j] = 0.0;
}

/*void initialize_vectormatrix_zero(int *B, size_t row, size_t col){
    for (size_t i=0;i<col;i++)
        for (size_t j=0;j<row;j++)
            B[i*row+j] = 0;
}*/

mwSize compute_fset(mwSize *fset, double *s_t, double *gradp, mwSize nb)
{
	/* Clean out fixed set first*/
	memset(fset, 0, sizeof(mwSize)*nb);
	mwSize fssize = 0;
	mwIndex jj;
	for (jj = 0; jj < nb; jj++) {
	if (s_t[jj] == 0.0 && gradp[jj] > 0.0)
		fset[fssize++] = jj;
	}
	return fssize;
}

mwSize checkTermination(mwSize *fset, double *gradp, double pgtol, mwSize nb)
{
	mwIndex term = 0;
	double pg = 0.0;
	mwSize ctr = 0;
    mwSize i;
	/* compute the norm of the gradient for all variables not in fset*/
	for (i = 0; i < nb; i++)
	{
		if (i==fset[ctr])
		{
			ctr+=1; 
			continue;
		}
		pg = fmax(pg, fabs(gradp[i]));
	}
	if (pg < pgtol)
		term = 2;
	return term;
}

double computeBBStep(double *scaledUtivals, mwSize *fset, double *oldgradp, double *Agp, double *Aagp, 
	mwSize fssize, mwIndex iter, ptrdiff_t RR, ptrdiff_t NB, ptrdiff_t pone, double one, double zero, 
	char *NOTRANS, char *TRANS)
{
	double step;
    mwSize i;
	for (i = 0; i < fssize; i++)
		oldgradp[fset[i]] = 0.0;
	dgemv(NOTRANS, &RR, &NB, &one, scaledUtivals, &RR, oldgradp, &pone, &zero, Agp, &pone);
	double root_Agp2 = dnrm2(&RR,Agp,&pone);
	double Agp2 = root_Agp2*root_Agp2;
	if ((iter-1) % 2 == 0)
	{
		double root_oldgradp2 = dnrm2(&NB,oldgradp,&pone);
		step = (root_oldgradp2*root_oldgradp2)/Agp2;
	}
	else
	{
		dgemv(TRANS, &RR, &NB, &one, scaledUtivals, &RR, Agp, &pone, &zero, Aagp, &pone);
		for (i = 0; i < fssize; i++)
			Aagp[fset[i]] = 0.0;
		double root_Aagp2 = dnrm2(&NB,Aagp,&pone);
		step = Agp2/(root_Aagp2*root_Aagp2);
	}
	return -step;
}


void mexFunction(
          int nlhs, mxArray *plhs[], 
		  int nrhs, const mxArray* prhs[])
{
	mwIndex ii;
	mwSize d, T, r;
    mxArray *ConstCell_Utxmx, *UomegaUomegaCellmx;
	double *maskvals, *Utvals, *ConstMatrixpr, *ConstCell_Utxpr, *UomegaUomegaCellpr;
    double *ConstMatrixpr_temp;
    mwIndex *CtIr, *CtJc;
    char *TRANS = "T";
    char *NOTRANS = "N";
	int maxNrThreads = 1;

    d = mxGetN(Ut);
    T = mxGetN(ConstMatrix);
    r = mxGetM(Ut);
    if(mxGetM(ConstMatrix) != r)
        mexErrMsgTxt("Dimensions mismatch in input arguments.");
    /*
    Cval = 1.0 / (2.0 * mxGetScalar(C));
    double const1 = 2.0*mxGetScalar(C);
    double const2 = -2.0*mxGetScalar(C);
    */
    int maxit = (int) mxGetScalar(maxinnerepoch);
    double myTolVal = 1.0*mxGetScalar(myTol);

    Utvals = mxGetPr(Ut);
    ConstMatrixpr = mxGetPr(ConstMatrix);
    CtIr = mxGetIr(maskS);
    CtJc = mxGetJc(maskS);
    mwSize nnzSvals = (mwSize) mxGetScalar(nnzS);

    plhs[0] = mxCreateDoubleMatrix(0,0,mxREAL);
    mxSetM(plhs[0],nnzSvals);
    mxSetN(plhs[0],1);
    mxSetPr(plhs[0], (double *)mxMalloc(sizeof(double)*nnzSvals));
    double *S = mxGetPr(plhs[0]);

    plhs[1] = mxCreateDoubleMatrix(0,0,mxREAL);
    mxSetM(plhs[1],r);
    mxSetN(plhs[1],T);
    mxSetPr(plhs[1], (double *)mxMalloc(sizeof(double)*r*T));
    double *UtZ_UtS = mxGetPr(plhs[1]);

    plhs[2] = mxCreateDoubleMatrix(0,0,mxREAL);
    mxSetM(plhs[2],r);
    mxSetN(plhs[2],T);
    mxSetPr(plhs[2], (double *)mxMalloc(sizeof(double)*r*T));
    double *UomegaUts_t = mxGetPr(plhs[2]);

    initialize_vectormatrix_zero(UomegaUts_t,r,T);
    initialize_vectormatrix_zero(S,nnzSvals,1);
    initialize_vectormatrix_zero(UtZ_UtS,r,T);

    ptrdiff_t pone = (ptrdiff_t) 1;
    mwIndex i;
    int id = 0;/*omp_get_thread_num();*/
    int nthrds = 1;/*omp_get_num_threads();*/

    int j, k, temp1, temp2;
    ptrdiff_t M, RR;
    double one = 1.0;
    double zero = 0.0;
    RR = (ptrdiff_t) r;
    /*M = (ptrdiff_t) d;*/

    mwSize *fset;
    fset = (mwSize*) malloc(sizeof(mwSize)*d);
    mwSize fssize, term;
    mwIndex iter, jj;
    double stepSize;
    mxArray *temp11 = mxCreateDoubleMatrix(r, 1, mxREAL);
    double *temp11p = mxGetPr(temp11);

    double *bp;
    double *UomegaUts_t_temp;
    double *s_t;
    mxArray *grad = mxCreateDoubleMatrix(d, 1, mxREAL);
    double *gradp = mxGetPr(grad);
    mxArray *oldgrad = mxCreateDoubleMatrix(d, 1, mxREAL);
    double *oldgradp = mxGetPr(oldgrad);
    mxArray *Ag = mxCreateDoubleMatrix(r, 1, mxREAL);
    double *Agp = mxGetPr(Ag);
    mxArray *Aag = mxCreateDoubleMatrix(d, 1, mxREAL);
    double *Aagp = mxGetPr(Aag);
    double objpart1, objpart2, objpart3, obj;

    mxArray *scaledUti = mxCreateDoubleMatrix(r, d, mxREAL);
    double *scaledUtivals = mxGetPr(scaledUti);
    mwIndex *Ctcol;
    mwSize nb;
    ptrdiff_t NB;

    for (i = id; i < T; i+=nthrds)
    {
        /*printf("%d, ", i);*/
        nb = CtJc[i+1]-CtJc[i];

        bp = UtZ_UtS + r*i;
        ConstMatrixpr_temp = ConstMatrixpr + r*i;
        memcpy(bp, ConstMatrixpr_temp, r*sizeof(double));
        if(nb > 0)
        {
            ptrdiff_t info = 0;

            Ctcol = CtIr+CtJc[i];
            s_t = S + CtJc[i];
            
            ConstCell_Utxmx = mxGetCell(ConstCell_Utx, i);
            ConstCell_Utxpr = mxGetPr(ConstCell_Utxmx);
            
            iter=0;
            for(j = 0; j < nb; ++j)
            {
                temp1 = j*r;
                temp2 = Ctcol[j]*r;
                for(k = 0; k < r; ++k)
                {
                    /* scaledUi(j, k) =  U(Ctcol[j], k) */
                    /*scaledUivals[j+k*m] = Uvals[Ctcol[j]+k*m];*/
                    scaledUtivals[k+temp1] = Utvals[k+temp2];
                }
            }

            NB = (ptrdiff_t) nb;
            dgemv(NOTRANS, &RR, &NB, &one, scaledUtivals, &RR, s_t, &pone, &zero, temp11p, &pone); /*useless step unless warm start*/
            dgemv(NOTRANS, &RR, &RR, &one, ConstCell_Utxpr, &RR, temp11p, &pone, &one, bp, &pone); /*useless step unless warm start*/
            dgemv(TRANS, &RR, &NB, &one, scaledUtivals, &RR, bp, &pone, &zero, gradp, &pone);

            memcpy(oldgradp, gradp, nb*sizeof(double));

    		fssize = compute_fset(fset, s_t, gradp, nb);

            for (iter=0; iter<maxit; iter++)
            {
                /*printf("iter: %d\n", iter);*/
            	/* check termination */
            	term = checkTermination(fset, gradp, myTolVal, nb);
            	if (term == 2)
            	{
            		break;
            	}
            	else if (iter >= maxit)
            	{
            		term = 1;
            		break;
            	}

            	/*compute step size*/
            	stepSize = computeBBStep(scaledUtivals, fset, oldgradp, Agp, Aagp, 
    				fssize, iter, RR, NB, pone, one, zero, NOTRANS, TRANS);

            	daxpy(&NB, &stepSize, gradp, &pone, s_t, &pone);
            	memcpy(oldgradp, gradp, nb*sizeof(double));

                /*projection step: can replace the 0 by an epsilon to truncate values close to 0*/
            	for (jj=0;jj<nb;jj++)
    				if (s_t[jj] < 0.0) 
    					s_t[jj] = 0.0;

                dgemv(NOTRANS, &RR, &NB, &one, scaledUtivals, &RR, s_t, &pone, &zero, temp11p, &pone);
                memcpy(bp, ConstMatrixpr_temp, r*sizeof(double));
                dgemv(NOTRANS, &RR, &RR, &one, ConstCell_Utxpr, &RR, temp11p, &pone, &one, bp, &pone);
                dgemv(TRANS, &RR, &NB, &one, scaledUtivals, &RR, bp, &pone, &zero, gradp, &pone);

    			fssize = compute_fset(fset, s_t, gradp, nb);
            }
            UomegaUomegaCellmx = mxGetCell(UomegaUomegaCell, i);
            UomegaUomegaCellpr = mxGetPr(UomegaUomegaCellmx);
            UomegaUts_t_temp = UomegaUts_t + r*i;
            dgemv(NOTRANS, &RR, &RR, &one, UomegaUomegaCellpr, &RR, temp11p, &pone, &zero, UomegaUts_t_temp, &pone);
            /*
            obj = ddot(&NB, &Z[Jc[i]], &pone, &Y_t[Jc[i]], &pone) - (Cval/2.0)*ddot(&NB, &Z[Jc[i]], &pone, &Z[Jc[i]], &pone) - 0.5*ddot(&RR, &UtZ_UtS[r*i], &pone, &UtZ_UtS[r*i], &pone);
            objpart1 = ddot(&NB, &Z[Jc[i]], &pone, &Y_t[Jc[i]], &pone);
            objpart2 = - (Cval/2.0)*ddot(&NB, &Z[Jc[i]], &pone, &Z[Jc[i]], &pone);
            objpart3 = - 0.5*ddot(&RR, &UtZ_UtS[r*i], &pone, &UtZ_UtS[r*i], &pone);
            */
            /*printf("task: %d, iter: %d, obj: %e\n",i,iter,obj);
            printf("%e, %e, %e\n", objpart1, objpart2, objpart3);*/
        }
    }
    free(fset);
    mxDestroyArray(temp11);
    mxDestroyArray(grad);
    mxDestroyArray(oldgrad);
    mxDestroyArray(Ag);
    mxDestroyArray(Aag);
    mxDestroyArray(scaledUti);
}
