function[Lopt, Ropt, infos, options, zopt, sopt, mask] = fixedrank_matrixcompletion_nnmc_new_objective(problem, Uinit, options)
    % Problem description
    d = problem.d;
    T = problem.T;
    r = problem.r;
    C = problem.C;
    I = problem.I; % Row indices
    J = problem.J; % Column indices
    Y = problem.Y; % Known entries
    
    mask = sparse(double(I), double(J), Y, d, T, length(I));
    [I, J, Y] = find(mask);
    I = uint32(I);
    J = uint32(J);
    iter = 0;
    
    length_entries = length(I); % Number of known entries
    if isfield(problem,'I_test') && ~isempty(problem.I_test)
        I_test = problem.I_test;
        J_test = problem.J_test;
        Y_test = problem.Y_test;
        length_entries_test = length(I_test);
    end
    
    % Some sparse structures that are often used.
    mask = sparse(double(I), double(J), ones(length(I), 1), d, T, length(I));
    z_mex = [];
    s_mex = [];
    UtZ_UtAtS = [];
    masks = [];
    
    % Local defaults for options
    localdefaults.maxiter = 50; % Max iterations.
    localdefaults.tolgradnorm = 1e-6; % Absolute tolerance on Gradnorm.
    localdefaults.maxinner = 30; % Max inner iterations for the tCG step.
    localdefaults.tolrelgradnorm = 1e-10; % Gradnorm/initGradnorm tolerance.
    localdefaults.method = 'cg'; % Default solver is trustregions (TR).
    localdefaults.verbosity = 2; % Default is print all the details.
    localdefaults.maxinnerepoch = 50; % Default max iteration for inner solver.
    localdefaults.innertolg = 1e-10; % Default norm(grad,'inf') tolerance for inner solver.
    
    % Manifold geometry
    problem.M = spectrahedronfactory(d, r);
    
    % Initialization
    if ~exist('Uinit', 'var')
        Uinit = [];
    end
    
    if ~exist('options', 'var') || isempty(options)
        options = struct();
    end
    options = mergeOptions(localdefaults, options);
    
    % Cost
    problem.cost = @cost;
    function [f, store] = cost(U, store)
        if ~isfield(store, 'z_mex_check')
%             nnls_timing = tic;
%             [z_mex, s_mex, UtZ_UtAtS, masks] = ...
%                 solve_nnmc_sparse_s(U, C, I, J, Y, mask, options.maxinnerepoch, options.innertolg, 1);
            
            [z_mex, s_mex, UtZ_UtAtS, masks] = ...
                solve_nnmc_sparse_s(U, C, I, J, Y, mask, ...
                options.maxinnerepoch, options.innertolg,options.sparse_s_flag,...
                options.num_omega_s,options.flag_parallel,options.numThreads);
            
            store.z_mex_check = true;
%             fprintf('solve_nnmc_sparse_s: ');
%             toc(nnls_timing);
            
        end
%         z = store.z_mex;
%         UtZ_UtAtS = store.UtZ_UtAtS_mex;
%         if any(isnan(z))
%             keyboard;
%         end
%         if any(isnan(UtZ_UtAtS(:)))
%             keyboard;
%         end
        f = (Y'*z_mex) - (z_mex'*z_mex)/(4*C) - 0.5*norm(UtZ_UtAtS,'fro')^2;
    end
    
    % Gradient
    problem.egrad = @egrad;
    function [grad, store] = egrad(U, store)
%         fprintf('egrad_start:\n');
%         egrad_timing = tic;
        if ~isfield(store, 'z_mex_check')
            [~, store] = cost(U, store);
        end
%         z = store.z_mex;
%         s = store.s_mex;
%         Is = store.Is;
%         Js = store.Js;
%         UtZ_UtAtS = store.UtZ_UtAtS_mex;
%         mask_time = tic;
%         ZS_sparse_mat = sparse([double(I);double(Is)], [double(J);double(Js)], [z;s], d, T);
%         toc(mask_time);
        grad = -mymultsparsefull(z_mex,UtZ_UtAtS',mask);
        grad = grad - mymultsparsefull(s_mex,UtZ_UtAtS',masks);
%         grad = -ZS_sparse_mat*UtZ_UtAtS';
%         grad = -(full(Z_sparse_mat)+S)*UtZ_UtAtS'; % Equivalent to grad = -(Z*ZtU).
%         if any(isnan(grad))
%             keyboard;
%         end
%         fprintf('egrad_time: ');
%         toc(egrad_timing);
    end
    
    %     % Hessian
    %     problem.ehess = @ehess;
    %     function [graddot, store] = ehess(U, Udot,store)
    %         if ~isfield(store, 'Z')
    %             [~, store] = cost(U, store);
    %         end
    %         Z = store.Z;
    %         UtZ = store.UtZ;
    %         Zchol = store.Zchol;
    %
    %         % Get Zdot at U and Udot.
    %         [Zdot, UtZdot, UdottZ] = squareLoss_dual_get_Zdot(U,C,I, J,mask,Udot,Z,UtZ,Zchol);
    %
    %         graddot = -mymultsparsefull(Zdot, UtZ', mask)...
    %             -mymultsparsefull(Z, (UtZdot+UdottZ)', mask);
    %     end
    
    % Stats that we compute every iteration
    options.statsfun = @mystatsfun;
    function stats = mystatsfun(~, U, stats, store)
        
        function output = AFUN(input,str)
            if strcmp(str,'notransp')
                output = L*(R*input);
            elseif strcmp(str,'transp')
                output = R'*(L'*input);
            else
                output = [];
                fprintf('wrong input to AFUN\n');
            end
        end
        
%         mystatsfun_timing = tic;
        %         primal_cost = stats.cost; % Equal to the partial dual cost.
        %         mysetsparseentries(Z_sparse_mat, store.Z);
        %         sigmax_Z = svds(Z_sparse_mat, 1); % Maximum singular value of Z.
        %         dual_cost = (Y'*store.Z) - (store.Z'*store.Z)/(4*C) - 0.5*sigmax_Z^2;
        %         stats.duality_gap = abs(primal_cost - dual_cost); % Absolute duality gap
        %         stats.rel_duality_gap = abs(stats.duality_gap/dual_cost); % Relative duality gap
        
        % Primal low-rank factors
        L = U;
        iter = iter + 1;
        refining_time = 0;
        if mod(iter,1) == 0
            refining = tic;
            [~, ~, UtZ_UtAtSref, ~] = ...
                solve_nnmc_sparse_s(L, C, I, J, Y, mask, ...
                options.finalmaxinnerepoch, options.innertolg, options.finalsparse_s_flag, ...
                options.finalnum_omega_s, options.flag_parallel, options.numThreads);
            refining_time = toc(refining);
            fprintf('refining_time: %e\n',refining_time);
            R = UtZ_UtAtSref;
            LR = L*R;
            fprintf('Iter :%d, refined: Minimum value: %e, Maximum value: %e\n',iter,min(LR(:)),max(LR(:)));
        else
            R = UtZ_UtAtS;
        end
        stats.refining_time = refining_time;
%         error_train = L*R;
%         fprintf('min: %e, max: %e\n',min(error_train(:)), max(error_train(:)));
        % Compute train RMSE
        error_train = myspmaskmult(L', R, I, J) - Y;
        %         error_train = myspmaskmult_parallel(L, R, I, J) ...
        %             - Y;
        stats.loss = norm(error_train)^2; % Square error on train data.
        stats.loss_rmse = sqrt((error_train'*error_train)/length_entries); % RMSE on train data.
        
        %stats.primal_objective = 0.5*norm(UtZ_UtAtS,'fro')^2 + C*stats.loss;
        primal_time = tic;
        [~,singularValues,~] = svds(@AFUN, [d T], r);
        stats.primal_objective = 0.5*(sum(diag(singularValues)))^2 + C*stats.loss;
        primal_time = toc(primal_time);
        fprintf('primal_time: %e\n',primal_time);
        
        if exist('I_test', 'var')
            error_test = myspmaskmult(L', R, I_test, J_test) - Y_test;
            %             error_test = myspmaskmult_parallel(L, R, I_test, J_test) ...
            %                 - Y_test;
            stats.loss_test = norm(error_test)^2; % Square error on test data.
            stats.loss_test_rmse = sqrt((error_test'*error_test)/length_entries_test); % RMSE on test data.
            fprintf('Primal objective: %e, LossRMSE %e TestLossRMSE %e \n', stats.primal_objective, stats.loss_rmse, stats.loss_test_rmse);
        end
    end
    
    
    % Additional stopping criteria
    options.stopfun = @mystopfun;
    function stopnow = mystopfun(problem, U, info, last)
        stopnow = (last >= 3 && info(last).gradnorm/info(1).gradnorm < options.tolrelgradnorm);
    end
    
    
    % Gradient and Hessian checks
%     checkgradient(problem);
%     pause;
    %     checkhessian(problem);
    %     pause;
    
    % Solver
    if strcmpi('TR', options.method)
        % Riemannian trustregions
        [Uopt,~,infos] = trustregions(problem, Uinit, options);
    elseif strcmpi('CG', options.method)
        % Riemannian conjugategradients
        options.beta_type = 'H-S';
        options.linesearch = @linesearch;
        options.ls_contraction_factor = .2;
        options.ls_optimism = 1.1;
        options.ls_suff_decr = 1e-4;
        options.ls_max_steps = 25;
        [Uopt,~,infos] = conjugategradient(problem, Uinit, options);
    end
    
    % Compute optimal Z
%     [zopt, Sopt, UtZ_UtAtSopt] = nnls_sbb(Uopt', Y, mask, C, options.maxinnerepoch, options.innertolg);
%     options.finalmaxinnerepoch
%     [zopt, sopt, UtZ_UtAtSopt, masksopt] = solve_nnmc_sparse_s(Uopt, C, I, J, Y, mask, options.maxinnerepoch, options.innertolg, 1);
        [zopt, sopt, UtZ_UtAtSopt, masksopt] = solve_nnmc_sparse_s(Uopt, C, I, J, Y, mask, ...
            options.maxinnerepoch, options.innertolg,options.sparse_s_flag,...
            options.num_omega_s,options.flag_parallel,options.numThreads);

    
    % Optimal low-rank factors
    Lopt = Uopt;
    Ropt = UtZ_UtAtSopt;
    
%     LR = Lopt*Ropt;
%     min(LR(:))

end