function [z,s,UtZ_UtAtS,masks] = solve_nnmc_sparse_s(U, C, I, J, Y, mask, maxinnerepoch, tolg, ...
                                    sparse_s_flag,num_omega_s,flag_parallel,numThreads)
    
    % compute U_omega_t'*y_omega_t
    d = size(mask,1);
    T = size(mask,2);
    Ut = U.';
%     time1 = tic;
    UomegatYomega = mymultfullsparse(Ut, Y, mask);
    %     fprintf('UomegatYomega: ');
%     t1 = toc(time1);
    
    % compute (U_omega_t'*U_omega_t) and Cholsky decomposition of (U_omega_t'*U_omega_t + I/2C)
%     time1 = tic;
    if flag_parallel
        [UomegaUomegaIcell, UomegaUomegaCell, ConstCell_Utx, ConstMatrix] = ...
            preprocess_parallel(mask, Ut, C, UomegatYomega, numThreads);
    else
        [UomegaUomegaIcell, UomegaUomegaCell, ConstCell_Utx, ConstMatrix] = ...
            preprocess(mask, Ut, C, UomegatYomega);
    end
    %     fprintf('rough1: ');
%     t2 = toc(time1);
    
    % compute S, U's+U_omega_t'z_t
%     time1 = tic;
    % find the sparsity pattern of S
    if sparse_s_flag==1
%         [Is,Js] = find(U*ConstMatrix < 0);
%         masks = sparse(double(Is), double(Js), ones(length(Is),1), d, T, length(Is));
        masks = double(sparse(U*ConstMatrix < 0));
    elseif sparse_s_flag == 2
        [Is, Js] = createRandMask(d, T, num_omega_s);
        masks = sparse(double(Is), double(Js), ones(length(Is),1), d, T, length(Is));
    else
%         [Is,Js] = find(U*ConstMatrix < Inf);
%         masks = sparse(double(Is), double(Js), ones(length(Is),1), d, T, length(Is));
        masks = sparse(ones(d,T));
    end
    lens = nnz(masks);
    %     fprintf('numS: %d, maxS: %d\n',lens,d*T);
    
%     t3 = toc(time1);
    
    
%     time1 = tic;
    if flag_parallel
        [s, UtZ_UtAtS, UomegaUts_t] = nnmc_new_sparse_s_parallel(Ut,ConstCell_Utx,ConstMatrix,...
            UomegaUomegaCell,maxinnerepoch,tolg,masks,lens,numThreads);
    else
        [s, UtZ_UtAtS, UomegaUts_t] = nnmc_new_sparse_s(Ut,ConstCell_Utx,ConstMatrix,...
            UomegaUomegaCell,maxinnerepoch,tolg,masks,lens);
    end
    %     fprintf('nnmc_new: ');
%     t4 = toc(time1);
    
    % compute z_t
%     time1 = tic;
    temp = UomegatYomega - UomegaUts_t;
    UtS = mymultfullsparse(Ut,s,masks);
    
    if flag_parallel
        UomegaUtS = myspmaskmult_parallel(Ut, UtS, I, J, numThreads);
        UtUinvUtY = blocksolve_parallel(UomegaUomegaIcell, temp, numThreads);
        UUtUinvUtY = myspmaskmult_parallel(Ut, UtUinvUtY, I, J, numThreads);
    else
        UomegaUtS = myspmaskmult(Ut, UtS, I, J);
        UtUinvUtY = blocksolve(UomegaUomegaIcell, temp);
        UUtUinvUtY = myspmaskmult(Ut, UtUinvUtY, I, J);
    end
    z = (2*C)*(Y - UomegaUtS - UUtUinvUtY);
%     t5 = toc(time1);
%     fprintf('UomegatYomega: %e, rough1: %e, masks: %e, nnmc_new: %e, z_t: %e\n',t1,t2,t3,t4,t5);
    clearvars -except z s UtZ_UtAtS masks;
    %     tic;
    %     temp2 = find(U*ConstMatrix < 0);
    %     toc;
    %     temp3 = find(S);
    %     temp4 = intersect(temp2,temp3);
    %     fprintf('%d, %d, %d, %d\n',length(temp2),length(temp3),length(temp4),length(temp4)/length(temp3));
    %     f = (Y'*z) - (z'*z)/(4*C) - 0.5*norm(UtZ_UtAtS,'fro')^2;
    %     if f < 0
    %         keyboard;
    %     end
    %     if max(Js) < T
    %         keyboard;
    %     end
end

function X = blocksolve_parallel(R, B, numThreads)
    % R is a cell containing n upper triangular square matrices of size r.
    % B is a matrix of size r x n. X is a matrix of size r x n where the i-th
    % column is the solution of the equation A{i}x = B(:, i), with
    % A{i} = R{i}.'*R{i}. This is: the R{i} matrices are the upper triangular
    % Cholesky factors of the matrices A{i} (which we do not require/store).
    
    X = mycholsolvecell_parallel(R, B, numThreads);    
end

function X = blocksolve(R, B)
    % R is a cell containing n upper triangular square matrices of size r.
    % B is a matrix of size r x n. X is a matrix of size r x n where the i-th
    % column is the solution of the equation A{i}x = B(:, i), with
    % A{i} = R{i}.'*R{i}. This is: the R{i} matrices are the upper triangular
    % Cholesky factors of the matrices Z{i} (which we do not require/store).
    
    X = mycholsolvecell(R, B);
end
