function [Zdot, UtZdot, UdottZ] = ...
    absoluteLoss_dual_get_Zdot_mex(U, myRidge, I_sparser, J_sparser, mask_sparser, ...
    mask, Udot, Z, UtZ, Ut_omega_sparse_U_omega_sparse_chol)

    Ut = U.';
    Udott = Udot.';
    UdottZ = mymultfullsparse(Udot.',Z,mask);
    numThreads = 4; % parallelizing some components of the inner problem over tasks. Number should be increased if system supports it. 
    parallel_flag = true;% default value should be true. It is advised to contact authors if one is not able to run the code with parallel_flag as true. 
    if parallel_flag
        RHS1 = myspmaskmult_parallel(Udott, UtZ, I_sparser, J_sparser, numThreads);
        RHS2 = myspmaskmult_parallel(Ut, UdottZ, I_sparser, J_sparser, numThreads);
    else
        RHS1 = myspmaskmult(Udott, UtZ, I_sparser, J_sparser);
        RHS2 = myspmaskmult(Ut, UdottZ, I_sparser, J_sparser);
    end
    RHS = -(RHS1 + RHS2);
    Ut_omega_sparse_RHS = mymultfullsparse(Ut, RHS, mask_sparser);
    if parallel_flag
        UtUinvUtRHS = blocksolve_parallel(Ut_omega_sparse_U_omega_sparse_chol, Ut_omega_sparse_RHS,...
            numThreads);
        UUtUinvUtRHS = myspmaskmult_parallel(Ut, UtUinvUtRHS, I_sparser, J_sparser, numThreads);
    else
        UtUinvUtRHS = blocksolve(Ut_omega_sparse_U_omega_sparse_chol, Ut_omega_sparse_RHS);
        UUtUinvUtRHS = myspmaskmult(Ut, UtUinvUtRHS, I_sparser, J_sparser);
    end
    Zdot = (1/myRidge)*(RHS - UUtUinvUtRHS);
    UtZdot = mymultfullsparse(Ut, Zdot, mask_sparser);
    clear UUtUinvUtRHS UtUinvUtRHS Ut_omega_sparse_RHS RHS RHS1 RHS2 Ut ...
        Ut_omega_sparse_U_omega_sparse_chol
end

function X = blocksolve_parallel(R, B, numThreads)
    % R is a cell containing n upper triangular square matrices of size r.
    % B is a matrix of size r x n. X is a matrix of size r x n where the i-th
    % column is the solution of the equation Z{i}x = B(:, i), with
    % Z{i} = R{i}.'*R{i}. This is: the R{i} matrices are the upper triangular
    % Cholesky factors of the matrices Z{i} (which we do not require/store).
    X = mycholsolvecell_parallel(R, B, numThreads);
    clear R B
end

function X = blocksolve(R, B)
    % R is a cell containing n upper triangular square matrices of size r.
    % B is a matrix of size r x n. X is a matrix of size r x n where the i-th
    % column is the solution of the equation Z{i}x = B(:, i), with
    % Z{i} = R{i}.'*R{i}. This is: the R{i} matrices are the upper triangular
    % Cholesky factors of the matrices Z{i} (which we do not require/store).
    X = mycholsolvecell(R, B);
    clear R B
end
