% This sample demo test code creates a synthetic matrix completion data set
% (with sparse outliers) of dimensions dxT and calls our second-order robust
% matrix completion algorithm, Proposed-cg-tr, to complete the matrix.

clear; clc; close all;
randomize_seed = 25;
rng(randomize_seed);
dataset = 'synthetic';

sp = 1;
fprintf('Dataset: %s, Split: %d of %d\n', dataset, sp, sp);
fprintf('randomize_seed: %d\n', randomize_seed);

% A synthetic problem instance.
d = 50; 
T = 500; 
truerank = 5;
OS = 5; % Over sampling. No of known entries relative to dimension. TYpicallY 4,5,8.
noiseFac = 0; 1e-6; % Noise factor
outlierFac = 0; 0.1; % Outlier factor
fprintf('d: %d, T: %d, truerank: %d, OS: %d, noiseFac: %e, outlierFac: %e\n',d,T,truerank,OS,noiseFac,outlierFac);

% Call function to create synthetic data set
[I, J, Y, I_test, J_test, Y_test] = createSyntheticDataWithOutliers(d, T, truerank, OS, noiseFac, outlierFac);
fprintf('Number of train instances: %d\n',length(Y));
fprintf('Number of test instances: %d\n',length(Y_test));

%% Create Problem structure and set hyper-parameters
% Set regularization parameter
C = 1e3;
fprintf('C: %e\n',C);
% Rank that we impose
rank = truerank;
fprintf('rank: %d\n',rank);

% Problem structure
problem.d = d;
problem.T = T;
problem.r = rank;
problem.C = C;
problem.I = I;
problem.J = J;
problem.Y = Y;
problem.I_test = I_test;
problem.J_test = J_test;
problem.Y_test = Y_test;

%% Proposed-cg-ab algorithm call

% Options are not mandatory
optstr.maxiter = 100; % Max iterations.
optstr.tolgradnorm = 1e-6; % Absolute tolerance on Gradnorm.
optstr.maxinner = 30; % Max inner iterations for the tCG step.
optstr.tolrelgradnorm = 1e-10; % Gradnorm/initGradnorm tolerance.
optstr.method = 'tr'; % option which ensures call to Proposed-tr-ab
optstr.innerepsilonsvr=0;
optstr.innerrelgradtol=1e-5;%1e-5
optstr.innermaxepoch=50;%50
optstr.innersolver=11;% 11. Required for l1-norm loss. Do not change this for trust regions. Currently, we only have trust regions code for l1-norm loss.
optstr.verbosity=2; % show the full output.

optstr

fprintf('------------------ Proposed-cg-ab ------------------\n')

% Run proposed algorithm
initproposedcgstart = tic;
% Initialization
mysparsemat = sparse(double(problem.I), double(problem.J), ones(length(problem.I), 1), problem.d, problem.T, length(problem.I));
mysetsparseentries(mysparsemat , problem.Y);
[Uinit, ~, ~]= svds(mysparsemat, problem.r);
Uinit = Uinit/norm(Uinit, 'fro');
initproposedcgtime = toc(initproposedcgstart);
fprintf('SVD based warm start\n');
fprintf('initproposedcgtime: %e\n',initproposedcgtime);

[Lopt, Ropt, Zopt, infoscg, optionscg] = fixedrank_robustmatrixcompletion(problem,Uinit,optstr);
% Wopt = Lopt * Ropt; % uncomment this line in order to get the complete matrix
proposed_cg_plot_rmse = [infoscg.loss_test_rmse];
proposed_cg_plot_time = [infoscg.time]+initproposedcgtime;
proposed_cg_plot_dualitygap = [infoscg.rel_duality_gap];
